@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyAboutModule.cpp,v 1.1 2005/07/17 16:06:04 chpe Exp $
+ *  $Id: EphyAboutModule.cpp,v 1.4 2005/07/27 22:16:53 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -276,7 +276,6 @@ EphyAboutModule::GetErrorMessage(nsIURI *aURI,
 				 host.get());
 		*aSecondary = _("The document could not be loaded because "
 				"encryption support is not installed.");
-		*aLinkIntro = _("There may be an old version of the page you wanted:");
 	}
 	else if (strcmp (aError, "netReset") == 0)
 	{
@@ -322,7 +321,7 @@ EphyAboutModule::GetErrorMessage(nsIURI *aURI,
 		 strcmp (aError, "proxyConnectFailure") == 0)
 	{
 		*aPrimary = g_strdup (_("Could not connect to proxy server."));
-		*aSecondary = _("Check the proxy server settings in Control Center. "
+		*aSecondary = _("Check your proxy server settings. "
 				"If the connection still fails, there may be "
 				"a problem with your proxy server or your "
 				"network connection.");
@@ -455,7 +454,7 @@ EphyAboutModule::CreateErrorPage(nsIURI *aErrorURI,
 		Write (stream, "<p>");
 		Write (stream, linkintro);
 		Write (stream, "<ul>\n");
-		Write (stream, "<li><a href=\"http://google.com/search?q=cache:");
+		Write (stream, "<li><a href=\"http://www.google.com/search?q=cache:");
 		Write (stream, rawurl.get());
 		Write (stream, "\">");
 		/* Translators: The text before the "|" is context to help you decide on
