@@ -15,16 +15,11 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: MozillaPrivate.h,v 1.8 2005/02/11 13:56:43 chpe Exp $
+ *  $Id: MozillaPrivate.h,v 1.10 2005/07/24 21:37:01 chpe Exp $
  */
 
 #include <glib.h>
 
-class nsIPrefBranch;
-
 namespace MozillaPrivate
 {
-	GList   *GetPrinterList ();
-
-	void	 SecureWallet (nsIPrefBranch *pref);
 }
