@@ -16,30 +16,46 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmark-action.c,v 1.57 2004/12/13 17:57:12 chpe Exp $
+ *  $Id: ephy-bookmark-action.c,v 1.66 2005/07/26 17:41:04 chpe Exp $
  */
 
 #include "config.h"
 
-#include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <gtk/gtktoolitem.h>
-
 #include "ephy-bookmark-action.h"
-#include "ephy-marshal.h"
-#include "ephy-dnd.h"
+#include "ephy-bookmarksbar-model.h"
 #include "ephy-bookmarksbar.h"
 #include "ephy-bookmarks.h"
+#include "ephy-link.h"
+#include "ephy-dnd.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-shell.h"
-#include "ephy-debug.h"
 #include "ephy-gui.h"
+#include "ephy-debug.h"
+
+#include <glib/gi18n.h>
+#include <gtk/gtkwidget.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkimagemenuitem.h>
+#include <gtk/gtkseparatormenuitem.h>
+#include <gtk/gtkmenushell.h>
+#include <gtk/gtkmenu.h>
+#include <gtk/gtktoolitem.h>
+#include <gtk/gtktoolbar.h>
+#include <gtk/gtkmain.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
 
 #include <string.h>
 
 /* FIXME tweak this, or make it configurable? (bug 148093) */
-#define ENTRY_WIDTH_CHARS	16
-#define TOOLITEM_WIDTH_CHARS	24
+#define ENTRY_WIDTH_CHARS	12
+#define TOOLITEM_WIDTH_CHARS	20
+#define LABEL_WIDTH_CHARS       32
 
 static void ephy_bookmark_action_init       (EphyBookmarkAction *action);
 static void ephy_bookmark_action_class_init (EphyBookmarkActionClass *class);
@@ -52,7 +68,7 @@ static GtkTargetEntry drag_targets[] =
 };
 static int n_drag_targets = G_N_ELEMENTS (drag_targets);
 
-struct EphyBookmarkActionPrivate
+struct _EphyBookmarkActionPrivate
 {
 	EphyNode *node;
 	gboolean smart_url;
@@ -72,17 +88,8 @@ enum
 	PROP_ICON
 };
 
-enum
-{
-	OPEN,
-	OPEN_IN_TAB,
-	LAST_SIGNAL
-};
-
 static GObjectClass *parent_class = NULL;
 
-static guint signals[LAST_SIGNAL] = { 0 };
-
 GType
 ephy_bookmark_action_get_type (void)
 {
@@ -103,10 +110,11 @@ ephy_bookmark_action_get_type (void)
 			(GInstanceInitFunc) ephy_bookmark_action_init,
 		};
 
-		type = g_type_register_static (GTK_TYPE_ACTION,
+		type = g_type_register_static (EPHY_TYPE_LINK_ACTION,
 					       "EphyBookmarkAction",
 					       &type_info, 0);
 	}
+
 	return type;
 }
 
@@ -115,9 +123,9 @@ create_tool_item (GtkAction *action)
 {
 	GtkWidget *item, *button, *hbox, *label, *icon, *entry;
 
-	LOG ("Creating tool item for action %p", action)
+	LOG ("Creating tool item for action %p", action);
 
-	item = (* GTK_ACTION_CLASS (parent_class)->create_tool_item) (action);
+	item = GTK_ACTION_CLASS (parent_class)->create_tool_item (action);
 
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox);
@@ -126,6 +134,7 @@ create_tool_item (GtkAction *action)
 	button = gtk_button_new ();
 	gtk_widget_add_events (GTK_WIDGET (button), GDK_BUTTON1_MOTION_MASK);
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+	gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
 	gtk_widget_show (button);
 	gtk_container_add (GTK_CONTAINER (hbox), button);
 	g_object_set_data (G_OBJECT (item), "button", button);
@@ -155,27 +164,25 @@ create_tool_item (GtkAction *action)
 }
 
 static void
-ephy_bookmark_action_sync_smart_url (GtkAction *action, GParamSpec *pspec, GtkWidget *proxy)
+ephy_bookmark_action_sync_smart_url (GtkAction *gaction, GParamSpec *pspec, GtkWidget *proxy)
 {
 	if (GTK_IS_TOOL_ITEM (proxy))
 	{
+		EphyBookmarkAction *action = EPHY_BOOKMARK_ACTION (gaction);
+		EphyBookmarkActionPrivate *priv = action->priv;
+		gboolean is_smart_url = priv->smart_url;
 		GtkWidget *entry, *icon;
-		gboolean smart_url;
+		guint width;
+
+		width = is_smart_url ? ephy_bookmarks_get_smart_bookmark_width (priv->node) : 0;
 
-		smart_url = EPHY_BOOKMARK_ACTION (action)->priv->smart_url;
 		entry = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "entry"));
 		icon = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "icon"));
 
-		if (smart_url)
-		{
-			gtk_widget_hide (icon);
-			gtk_widget_show (entry);
-		}
-		else
-		{
-			gtk_widget_show (icon);
-			gtk_widget_hide (entry);
-		}
+		g_object_set (icon, "visible", !is_smart_url, NULL);
+		g_object_set (entry, "visible", is_smart_url, NULL);
+		gtk_entry_set_width_chars (GTK_ENTRY (entry),
+					   width > 0 ? width : ENTRY_WIDTH_CHARS);
 	}
 }
 
@@ -298,87 +305,82 @@ ephy_bookmark_action_sync_label (GtkAction *gaction,
 }
 
 static void
-open_in_tab_activate_cb (GtkWidget *widget, EphyBookmarkAction *action)
+open_in_tab_activate_cb (GtkWidget *widget,
+			 EphyBookmarkAction *action)
 {
-	const char *url;
+	EphyBookmarkActionPrivate *priv = action->priv;
+	EphyBookmarks *bookmarks;
+	const char *location;
+	char *address;
 
-	g_return_if_fail (action->priv->node != NULL);
+	g_return_if_fail (priv->node != NULL);
 
-	url = ephy_node_get_property_string (action->priv->node,
-					     EPHY_NODE_BMK_PROP_LOCATION);
-	g_signal_emit (action, signals[OPEN_IN_TAB], 0, url, FALSE);
+	location = ephy_node_get_property_string
+			(priv->node, EPHY_NODE_BMK_PROP_LOCATION);
+	g_return_if_fail (location != NULL);
+
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell_get_default ());
+	address = ephy_bookmarks_resolve_address (bookmarks, location, NULL);
+	g_return_if_fail (address != NULL);
+
+	ephy_link_open (EPHY_LINK (action), address, NULL,
+			EPHY_LINK_NEW_TAB | EPHY_LINK_JUMP_TO);
+
+	g_free (address);
 }
 
 static void
 open_in_window_activate_cb (GtkWidget *widget, EphyBookmarkAction *action)
 {
-	const char *url;
+	EphyBookmarkActionPrivate *priv = action->priv;
+	EphyBookmarks *bookmarks;
+	const char *location;
+	char *address;
 
-	g_return_if_fail (action->priv->node != NULL);
+	g_return_if_fail (priv->node != NULL);
 
-	url = ephy_node_get_property_string (action->priv->node,
-					     EPHY_NODE_BMK_PROP_LOCATION);
-	g_signal_emit (action, signals[OPEN_IN_TAB], 0, url, TRUE);
+	location = ephy_node_get_property_string
+			(priv->node, EPHY_NODE_BMK_PROP_LOCATION);
+	g_return_if_fail (location != NULL);
+
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell_get_default ());
+	address = ephy_bookmarks_resolve_address (bookmarks, location, NULL);
+	g_return_if_fail (address != NULL);
+
+	ephy_link_open (EPHY_LINK (action), address, NULL, EPHY_LINK_NEW_WINDOW);
+
+	g_free (address);
 }
 
 static void
 activate_cb (GtkWidget *widget,
 	     EphyBookmarkAction *action)
 {
-	const char *url;
-	char *location = NULL, *text = NULL;
+	EphyBookmarkActionPrivate *priv = action->priv;
+	EphyBookmarks *bookmarks;
+	const char *location;
+	char *address, *text = NULL;
 
-	g_return_if_fail (action->priv->node != NULL);
+	g_return_if_fail (priv->node != NULL);
 
-	url = ephy_node_get_property_string (action->priv->node,
-					     EPHY_NODE_BMK_PROP_LOCATION);
+	location = ephy_node_get_property_string
+			(priv->node, EPHY_NODE_BMK_PROP_LOCATION);
+	g_return_if_fail (location != NULL);
 
 	if (GTK_IS_EDITABLE (widget))
 	{
 		text = gtk_editable_get_chars (GTK_EDITABLE (widget), 0, -1);
 	}
 
-	if (text != NULL && text[0] != '\0')
-	{
-		EphyBookmarks *bookmarks;
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell_get_default ());
 
-		bookmarks = ephy_shell_get_bookmarks (ephy_shell);
+	address = ephy_bookmarks_resolve_address (bookmarks, location, text);
+	g_return_if_fail (address != NULL);
 
-		location = ephy_bookmarks_solve_smart_url (bookmarks,
-							   url,
-							   text);
-	}
-	else
-	{
-		if (action->priv->smart_url)
-		{
-			GnomeVFSURI *uri;
+	ephy_link_open (EPHY_LINK (action), address, NULL,
+			ephy_gui_is_middle_click () ? EPHY_LINK_NEW_TAB : 0);
 
-			uri = gnome_vfs_uri_new (url);
-
-			if (uri)
-			{
-				location = g_strdup (gnome_vfs_uri_get_host_name (uri));
-				gnome_vfs_uri_unref (uri);
-			}
-		}
-
-		if (location == NULL)
-		{
-			location = g_strdup (url);
-		}
-	}
-
-	if (ephy_gui_is_middle_click ())
-	{
-		g_signal_emit (action, signals[OPEN_IN_TAB], 0, location, FALSE);
-	}
-	else
-	{
-		g_signal_emit (action, signals[OPEN], 0, location);
-	}
-
-	g_free (location);
+	g_free (address);
 	g_free (text);
 }
 
@@ -397,7 +399,8 @@ drag_data_get_cb (GtkWidget *widget, GdkDragContext *context,
 		  GtkSelectionData *selection_data, guint info,
 		  guint32 time, EphyBookmarkAction *action)
 {
-	const char *address;
+	const char *address, *title;
+	char *data;
 
 	g_return_if_fail (action->priv->node != NULL);
 
@@ -405,8 +408,14 @@ drag_data_get_cb (GtkWidget *widget, GdkDragContext *context,
 						 EPHY_NODE_BMK_PROP_LOCATION);
 	g_return_if_fail (address != NULL);
 
+	title = ephy_node_get_property_string (action->priv->node,
+					       EPHY_NODE_BMK_PROP_TITLE);
+	g_return_if_fail (title != NULL);
+
+	data = g_strdup_printf ("%s\n%s", address, title);
 	gtk_selection_data_set (selection_data, selection_data->target, 8,
-				(unsigned char *) address, strlen (address));
+				(unsigned char *) data, strlen (data));
+	g_free (data);
 }
 
 static int
@@ -483,8 +492,9 @@ drag_motion_cb (GtkWidget *widget, GdkEventMotion *event, EphyBookmarkAction *ac
 		target_list = gtk_target_list_new (drag_targets, n_drag_targets);
 
 		stop_drag_check (action, widget);
-		gtk_drag_begin (widget, target_list, GDK_ACTION_MOVE |
-				GDK_ACTION_COPY, 1, (GdkEvent*)event);
+		gtk_drag_begin (widget, target_list, GDK_ACTION_ASK |
+				GDK_ACTION_MOVE | GDK_ACTION_COPY, 1,
+				(GdkEvent*)event);
 
 		gtk_target_list_unref (target_list);
 	}
@@ -531,7 +541,7 @@ show_context_menu (EphyBookmarkAction *action,
 		   GdkEventButton *event,
 		   GtkMenuPositionFunc func)
 {
-	GtkWidget *menu, *item;
+	GtkWidget *menu, *item, *image;
 	gboolean last;
 
 	menu = gtk_menu_new ();
@@ -563,8 +573,11 @@ show_context_menu (EphyBookmarkAction *action,
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_REMOVE, NULL);
+	item = gtk_image_menu_item_new_with_mnemonic (_("_Remove from Toolbar"));
 	gtk_widget_show (item);
+	image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	gtk_widget_show (image);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	g_signal_connect (item, "activate",
 			  G_CALLBACK (remove_activate_cb), proxy);
@@ -663,9 +676,9 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 {
 	GtkWidget *button, *entry;
 
-	LOG ("Connecting action %p to proxy %p", action, proxy)
+	LOG ("Connecting action %p to proxy %p", action, proxy);
 
-	(* GTK_ACTION_CLASS (parent_class)->connect_proxy) (action, proxy);
+	GTK_ACTION_CLASS (parent_class)->connect_proxy (action, proxy);
 
 	ephy_bookmark_action_sync_icon (action, NULL, proxy);
 	g_signal_connect_object (action, "notify::icon",
@@ -702,7 +715,11 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 		GtkLabel *label;
 
 		label = (GtkLabel *) ((GtkBin *) proxy)->child;
+
 		gtk_label_set_use_underline (label, FALSE);
+		gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
+		gtk_label_set_max_width_chars (label, LABEL_WIDTH_CHARS);
+
 		g_signal_connect (proxy, "activate", G_CALLBACK (activate_cb), action);
 	}
 }
@@ -821,7 +838,7 @@ ephy_bookmark_action_finalize (GObject *object)
 {
  /*       EphyBookmarkAction *eba = EPHY_BOOKMARK_ACTION (object);*/
 
-	LOG ("Bookmark action %p finalized", object)
+	LOG ("Bookmark action %p finalized", object);
 
 	parent_class->finalize (object);
 }
@@ -843,29 +860,6 @@ ephy_bookmark_action_class_init (EphyBookmarkActionClass *class)
 	object_class->set_property = ephy_bookmark_action_set_property;
 	object_class->get_property = ephy_bookmark_action_get_property;
 
-	signals[OPEN] =
-                g_signal_new ("open",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (EphyBookmarkActionClass, open),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__STRING,
-                              G_TYPE_NONE,
-                              1,
-			      G_TYPE_STRING);
-
-	signals[OPEN_IN_TAB] =
-                g_signal_new ("open_in_tab",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (EphyBookmarkActionClass, open_in_tab),
-                              NULL, NULL,
-                              ephy_marshal_VOID__STRING_BOOLEAN,
-                              G_TYPE_NONE,
-                              2,
-			      G_TYPE_STRING,
-			      G_TYPE_BOOLEAN);
-
 	g_object_class_install_property (object_class,
                                          PROP_BOOKMARK,
 					 g_param_spec_pointer ("bookmark",
