@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.156.2.1 2005/03/12 23:18:48 chpe Exp $
+ *  $Id: window-commands.c,v 1.165 2005/08/01 19:28:53 chpe Exp $
  */
 
 #include "config.h"
@@ -28,7 +28,6 @@
 #include "ephy-debug.h"
 #include "ephy-command-manager.h"
 #include "window-commands.h"
-#include "print-dialog.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
 #include "ephy-embed-prefs.h"
@@ -44,6 +43,8 @@
 #include "ephy-zoom.h"
 #include "ephy-notebook.h"
 #include "ephy-toolbar-editor.h"
+#include "ephy-find-toolbar.h"
+#include "ephy-location-entry.h"
 
 #include <string.h>
 #include <glib.h>
@@ -58,13 +59,6 @@
 #include <gtk/gtktoggleaction.h>
 #include <glib/gi18n.h>
 
-void
-window_cmd_edit_find (GtkAction *action,
-		      EphyWindow *window)
-{
-	ephy_window_find (window);
-}
-
 void
 window_cmd_file_print_setup (GtkAction *action,
 			     EphyWindow *window)
@@ -110,7 +104,7 @@ window_cmd_go_back (GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_embed_activate (embed);
+	gtk_widget_grab_focus (GTK_WIDGET (embed));
 
 	ephy_embed_go_back (embed);
 }
@@ -124,7 +118,7 @@ window_cmd_go_up (GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_embed_activate (embed);
+	gtk_widget_grab_focus (GTK_WIDGET (embed));
 
 	ephy_embed_go_up (embed);
 }
@@ -135,10 +129,8 @@ window_cmd_file_send_to	(GtkAction *action,
 {
 	EphyTab *tab;
 	EphyEmbed *embed;
-	char *url;
-	char *embed_location;
-	char *location;
-	char *title;
+	const char *address;
+	char *url, *location, *title;
 
 	tab = ephy_window_get_active_tab (window);
 	g_return_if_fail (tab != NULL);
@@ -146,9 +138,8 @@ window_cmd_file_send_to	(GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	embed_location = ephy_embed_get_location (embed, TRUE);
-	location = gnome_vfs_escape_string (embed_location);
-	g_free (embed_location);
+	address = ephy_tab_get_address (tab);
+	location = gnome_vfs_escape_string (address);
 
 	title = ephy_embed_get_title (embed);
 	if (title != NULL)
@@ -182,7 +173,7 @@ window_cmd_go_forward (GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_embed_activate (embed);
+	gtk_widget_grab_focus (GTK_WIDGET (embed));
 
 	ephy_embed_go_forward (embed);
 }
@@ -199,11 +190,11 @@ window_cmd_view_stop (GtkAction *action,
 		      EphyWindow *window)
 {
 	EphyEmbed *embed;
-
+	
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_embed_activate (embed);
+	gtk_widget_grab_focus (GTK_WIDGET (embed));
 
 	ephy_embed_stop_load (embed);
 }
@@ -243,7 +234,7 @@ window_cmd_view_reload (GtkAction *action,
 		force = TRUE;
 	}
 
-	ephy_embed_activate (embed);
+	gtk_widget_grab_focus (GTK_WIDGET (embed));
 
 	ephy_embed_reload (embed, force);
 }
@@ -297,9 +288,8 @@ window_cmd_file_bookmark_page (GtkAction *action,
 	EphyEmbed *embed;
 	EphyBookmarks *bookmarks;
 	GtkWidget *new_bookmark;
-	char *location;
-	const char *icon;
-	char *title = NULL;
+	const char *location, *icon;
+	char *title;
 
 	tab = ephy_window_get_active_tab (window);
 	g_return_if_fail (tab != NULL);
@@ -307,7 +297,7 @@ window_cmd_file_bookmark_page (GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	location = ephy_embed_get_location (embed, TRUE);
+	location = ephy_tab_get_address (tab);
 
 	title = ephy_embed_get_title (embed);
 	if (title == NULL)
@@ -331,7 +321,6 @@ window_cmd_file_bookmark_page (GtkAction *action,
 	}
 
 	g_free (title);
-	g_free (location);
 }
 
 static void
@@ -555,28 +544,34 @@ window_cmd_edit_select_all (GtkAction *action,
 	}
 }
 
+void
+window_cmd_edit_find (GtkAction *action,
+		      EphyWindow *window)
+{
+	EphyFindToolbar *toolbar;
+	
+	toolbar = EPHY_FIND_TOOLBAR (ephy_window_get_find_toolbar (window));
+	ephy_find_toolbar_open (toolbar, FALSE, FALSE);
+}
+
 void
 window_cmd_edit_find_next (GtkAction *action,
 			   EphyWindow *window)
 {
-	EphyEmbed *embed;
+	EphyFindToolbar *toolbar;
 
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	ephy_embed_find_next (embed, FALSE);
+	toolbar = EPHY_FIND_TOOLBAR (ephy_window_get_find_toolbar (window));
+	ephy_find_toolbar_find_next (toolbar);
 }
 
 void
 window_cmd_edit_find_prev (GtkAction *action,
 			   EphyWindow *window)
 {
-	EphyEmbed *embed;
+	EphyFindToolbar *toolbar;
 
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	ephy_embed_find_next (embed, TRUE);
+	toolbar = EPHY_FIND_TOOLBAR (ephy_window_get_find_toolbar (window));
+	ephy_find_toolbar_find_previous (toolbar);
 }
 
 void
@@ -676,19 +671,21 @@ void
 window_cmd_view_page_source (GtkAction *action,
 			     EphyWindow *window)
 {
+	EphyTab *tab;
 	EphyEmbed *embed;
-	char *address;
-	char *scheme;
+	const char *address;
 	guint32 user_time;
 
+	tab = ephy_window_get_active_tab (window);
+	g_return_if_fail (tab != NULL);
+
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	address = ephy_embed_get_location (embed, TRUE);
-	scheme = gnome_vfs_get_uri_scheme (address);
+	address = ephy_tab_get_address (tab);
 	user_time = gtk_get_current_event_time ();
 
-	if (strcmp (scheme, "file") == 0)
+	if (g_str_has_prefix (address, "file://"))
 	{
 		ephy_file_launch_handler ("text/plain", address, user_time);
 	}
@@ -696,9 +693,6 @@ window_cmd_view_page_source (GtkAction *action,
 	{
 		save_temp_source (embed, user_time);
 	}
-
-	g_free (scheme);
-	g_free (address);
 }
 
 void
@@ -753,25 +747,39 @@ void
 window_cmd_help_about (GtkAction *action,
 		       GtkWidget *window)
 {
-	static char *authors[] = {
-		"Marco Pesenti Gritti <marco@gnome.org>",
-		"Xan Lopez <xan@gnome.org>",
-		"David Bordoley <bordoley@msu.edu>",
-		"Christian Persch <chpe@gnome.org>",
-		NULL
+	const char * const authors[] = {
+		"Marco Pesenti Gritti",
+		"Adam Hooper",
+		"Xan Lopez",
+		"Christian Persch",
+		"Jean-François Rameau",
+		"",
+		_("Contact us at:"),
+		"<epiphany-list@gnome.org>",
+		"",
+		_("Contributors:"),
+		"Crispin Flowerday",
+		"Raphaël Slinckx",
+		"",
+		_("Past developers:"),
+		"David Bordoley",
+		NULL,
 	};
-
-	static char *documenters[] = {
-		"Patanjali Somayaji <patanjali@codito.com>",
-		"David Bordoley <bordoley@msu.edu>",
-		"Piers Cornwell <piers@gnome.org>", 
+	const char * const documenters[] = {
+		"Piers Cornwell", 
+		"Patanjali Somayaji",
+		"David Bordoley",
+		"",
+		_("Contact us at:"),
+		"<epiphany-list@gnome.org> or <gnome-doc-list@gnome.org>",
 		NULL
 	};
 
 	gtk_show_about_dialog (GTK_WINDOW (window),
-			       "name", _("Epiphany"),
+			       "name", _("Epiphany Web Browser"),
 			       "version", VERSION,
-			       "copyright", "Copyright \xc2\xa9 2002-2005 Marco Pesenti Gritti",
+			       "copyright", "Copyright © 2002-2004 Marco Pesenti Gritti\n"
+					    "Copyright © 2003-2005 The Epiphany Developers",
 			       "authors", authors,
 			       "documenters", documenters,
 				/* Translators: This is a special message that shouldn't be translated
@@ -912,3 +920,25 @@ window_cmd_browse_with_caret (GtkAction *action,
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 	eel_gconf_set_boolean (CONF_BROWSE_WITH_CARET, active);
 }
+
+#ifdef HAVE_X11_XF86KEYSYM_H
+void
+window_cmd_go_home (GtkAction *action,
+		    EphyWindow *window)
+{
+	char *location;
+
+	location = eel_gconf_get_string (CONF_GENERAL_HOMEPAGE);
+
+	if (location == NULL || location[0] == '\0')
+	{
+		g_free (location);
+
+		location = g_strdup ("about:blank");
+	}
+
+	ephy_window_load_url (window, location);
+
+	g_free (location);
+}
+#endif /* HAVE_X11_XF86KEYSYM_H */
